function generate = QPproblem(setname)

switch lower(setname)


   % -----------------------------------------------------
   % A - 100x200 Gaussian matrix (lambda)
   % -----------------------------------------------------
   case {'a'}
      generate  = @QPbatchSetA;

   % -----------------------------------------------------
   % B - 300x2000 Gaussian matrix (lambda)
   % -----------------------------------------------------
   case {'b'}
      generate  = @QPbatchSetB;

   % -----------------------------------------------------
   % C - 300x2048 Restricted DCT (lambda)
   % -----------------------------------------------------
   case {'c'}
      generate  = @QPbatchSetC;

   % -----------------------------------------------------
   % D - 1200x8192 Restricted DCT (tau)
   % -----------------------------------------------------
   case {'d'}
      generate  = @QPbatchSetD;

   % -----------------------------------------------------
   % E - 4800x32768 Restricted DCT (tau)
   % -----------------------------------------------------
   case {'e'}
      generate  = @QPbatchSetE;

   % -----------------------------------------------------
   % F - 9600x65536 Restricted DCT (tau)
   % -----------------------------------------------------
   case {'f'}
      generate  = @QPbatchSetF;

   % -----------------------------------------------------
   % Unknown problem set - error
   % -----------------------------------------------------
   otherwise
      error('Unknown problem set');
end



function [A,b,x0,groups,weights,lambda,tau] = QPbatchSetA(idx)

   randState  = rand('state');
   randnState = randn('state');
   rand('state',idx);
   randn('state',idx);

   groups  = kron(1:20,ones(1,10))';
   weights = 1+rand(20,1);
   lambda  = 10;
   tau     = NaN;
   A  = randn(100,200);
   p  = randperm(20) - 1;
   p  = p(1:3);
   x0 = zeros(200,1);

   for i=1:length(p)
      x0(p(i)*10 + (1:10)) = rand(10,1);
   end

   b = A*x0;

   randn('state',randnState);
   rand('state' ,randState);



function [A,b,x0,groups,weights,lambda,tau] = QPbatchSetB(idx)

   randState  = rand('state');
   randnState = randn('state');
   rand('state',idx);
   randn('state',idx);

   groups  = kron(1:200,ones(1,10))';
   weights = 1+rand(200,1);
   lambda  = 10;
   tau     = NaN;
   A  = randn(300,2000);
   p  = randperm(200) - 1;
   p  = p(1:5);
   x0 = zeros(2000,1);

   for i=1:length(p)
      x0(p(i)*10 + (1:10)) = rand(10,1);
   end

   b = A*x0;

   randn('state',randnState);
   rand('state' ,randState);

   
   
function [A,b,x0,groups,weights,lambda,tau] = QPbatchSetC(idx)

   randState  = rand('state');
   randnState = randn('state');
   rand('state',idx);
   randn('state',idx);

   ngroups = 256;
   groups  = kron(1:ngroups,ones(1,8))';
   weights = ones(ngroups,1);
   lambda  = 0.01;
   tau     = NaN;
   r       = randperm(2048);
   r       = r(1:300);
   A       = @(x,mode) FunRestrictedDCT(2048,r,x,mode);
   p       = randperm(ngroups) - 1;
   p       = p(1:7);
   x0      = zeros(2048,1);

   for i=1:length(p)
      x0(p(i)*8 + (1:8)) = randn(8,1);
   end

   b = A(x0,1);

   randn('state',randnState);
   rand('state' ,randState);

   
function [A,b,x0,groups,weights,lambda,tau] = QPbatchSetD(idx)

   randState  = rand('state');
   randnState = randn('state');
   rand('state',idx);
   randn('state',idx);

   ngroups = 512;
   groups  = kron(1:ngroups,ones(1,16))';
   weights = ones(ngroups,1);
   r       = randperm(8192);
   r       = r(1:1200);
   A       = @(x,mode) FunRestrictedDCT(8192,r,x,mode);
   p       = randperm(ngroups) - 1;
   p       = p(1:10);
   x0      = zeros(8192,1);

   for i=1:length(p)
      x0(p(i)*16 + (1:16)) = randn(16,1);
   end

   lambda = NaN;
   tau    = sum(weights.*sqrt(accumarray(groups,x0.^2)));
   b      = A(x0,1) + randn(1200,1) * 1e-3;

   randn('state',randnState);
   rand('state' ,randState);

   
function [A,b,x0,groups,weights,lambda,tau] = QPbatchSetE(idx)

   randState  = rand('state');
   randnState = randn('state');
   rand('state',idx);
   randn('state',idx);

   ngroups = 512;
   groups  = kron(1:ngroups,ones(1,64))';
   weights = ones(ngroups,1);
   r       = randperm(32768);
   r       = r(1:4800);
   A       = @(x,mode) FunRestrictedDCT(32768,r,x,mode);
   p       = randperm(ngroups) - 1;
   p       = p(1:10);
   x0      = zeros(32768,1);

   for i=1:length(p)
      x0(p(i)*64 + (1:64)) = randn(64,1);
   end

   lambda = NaN;
   tau    = sum(weights.*sqrt(accumarray(groups,x0.^2)));
   b      = A(x0,1) + randn(4800,1) * 1e-2;

   randn('state',randnState);
   rand('state' ,randState);

function [A,b,x0,groups,weights,lambda,tau] = QPbatchSetF(idx)

   randState  = rand('state');
   randnState = randn('state');
   rand('state',idx);
   randn('state',idx);

   ngroups = 512;
   groups  = kron(1:ngroups,ones(1,128))';
   weights = ones(ngroups,1);
   r       = randperm(65536);
   r       = r(1:9600);
   A       = @(x,mode) FunRestrictedDCT(65536,r,x,mode);
   p       = randperm(ngroups) - 1;
   p       = p(1:8);
   x0      = zeros(65536,1);

   for i=1:length(p)
      x0(p(i)*128 + (1:128)) = randn(128,1);
   end

   lambda = NaN;
   tau    = sum(weights.*sqrt(accumarray(groups,x0.^2)));
   b      = A(x0,1) + randn(9600,1) * 1e-2;

   randn('state',randnState);
   rand('state' ,randState);

function y = FunRestrictedDCT(n,r,x,mode)
if mode == 1
   z = dct(x);
   y = z(r);
else
   z = zeros(n,1);
   z(r) = x;
   y = idct(z);
end
