function [w] = UGM_SGD(funObj,w,options,X,Xedge,y,varargin)
% options.mu : learning rate (
% options.batchSize : size of mini-batches
% maxIter : number of mini-batch updates to perform
% tuneSize : size of tuning set 

[nInstances,nFeatures,nNodes] = size(X);

[mu,batchSize] = myProcessOptions(options,...
    'mu',.01,'batchSize',10);

[maxIter,tuneSize] = myProcessOptions(options,...
    'maxIter',ceil(250*nInstances/batchSize),'tuneSize',nInstances/10);

[tuneIters] = myProcessOptions(options,'tuneIters',maxIter/10);

SGD_opts = struct('batchSize',batchSize,'mu',mu,'maxIter',maxIter);
if length(mu) > 1
   fprintf('Tuning\n'); 
   
   tuneSet = ceil(rand(tuneSize,1)*nInstances);
   for m = 1:length(mu)
       fprintf('Testing mu = %f\n',mu(m));
       SGD_opts.mu = mu(m);
       SGD_opts.maxIter = tuneIters;
       wTune(:,m) = SGD(funObj,w,SGD_opts,X(tuneSet,:,:),Xedge(tuneSet,:,:),y(tuneSet,:),varargin{:});
       err(m) = funObj(wTune(:,m),X(tuneSet,:,:),Xedge(tuneSet,:,:),y(tuneSet,:),varargin{:});
   end
   [minErr,minPos] = min(err);
   SGD_opts.mu = mu(minPos);
   SGD_opts.maxIter = maxIter;
end

fprintf('Performing SGD with mu = %f\n',SGD_opts.mu);
w = SGD(funObj,w,SGD_opts,X,Xedge,y,varargin{:});

end

function [w] = SGD(funObj,w,SGD_opts,X,Xedge,y,varargin);

[nInstances,nFeatures,nNodes] = size(X);
[batchSize,mu,maxIter] = deal(SGD_opts.batchSize,SGD_opts.mu,SGD_opts.maxIter);

for i = 1:maxIter

    % Select a random batch
    batch = ceil(rand(batchSize,1)*nInstances);
    
    % Compute Gradient of batch
    [f,g] = funObj(w,X(batch,:,:),Xedge(batch,:,:),y(batch,:),varargin{:});

    if mod(i,100) == 0
        fprintf('Iter = %d, E[passes] = %d, f_b = %.3f, opt_b = %.3f\n',i,floor(i*batchSize/nInstances),f,sum(abs(g)));
    end

    % Take step in negative gradient direction
    w = w - mu*g;

end
end