function  [y] = UGM_Decode_Block_SA(nodePot, edgePot, edgeStruct, infoStruct, blocks)
% INPUT
% nodePot(node,class)
% edgePot(class,class,edge) where e is referenced by V,E (must be the same
% between feature engine and inference engine)
%
% OUTPUT
% nodeLabel(node)

[nNodes,maxStates] = size(nodePot);
nEdges = size(edgePot,3);
edgeEnds = edgeStruct.edgeEnds;
V = edgeStruct.V;
E = edgeStruct.E;
nStates = infoStruct.nStates;
maxIter = infoStruct.maxIter;

% Initialize
[junk y] = max(nodePot,[],2);

% Compute Potential of Configuration
pot_current = UGM_ConfigurationPotential(y,nodePot,edgePot,edgeEnds);
maxPot = pot_current;
nodeLabels = y;

% Compute rate of temperature decrease
smallProb = 1e-5;
Trate = exp(log(smallProb)/maxIter);

T = 1;
for i = 1:maxIter
    
    % Update Temperature
    T = T*Trate;
    
    for b = 1:length(blocks)
        
        y_old = y;
        
        % Generate Proposal
        for n = blocks{b}
            y(n) = ceil(nStates(n)*rand);
        end
        
        if y(blocks{b}) ~= y_old(blocks{b})
            % Compute Potential of Configuration
            pot_new = UGM_ConfigurationPotential(y,nodePot,edgePot,edgeEnds);

            % Keep if increases potential
            if pot_new > pot_current
                pot_current = pot_new;

                % Record Max
                if pot_current > maxPot
                    maxPot = pot_current;
                    nodeLabels = y;
                end
            elseif rand < T
                % Still keep with some temperature-dependent probability
                pot_current = pot_new;
            else
                % Otherwise Reject
                y = y_old;
            end
        end
        
    end
end