clear all
close all

ising = 1;
tied = 1;
useMex = 1;
maxIter = 250;

doGibbs = 0; % Slower than ICM
doLoopy = 0; % Slower than Gibbs

% Choose 1 of the below for learning strategy
doPseudo = 0;
doViterbi = 0; % Slower than Pseudolikelihood (should use a stochastic optimizer)
doLoopyLearn = 1; % Much Slower than Pseudolikelihood (loopy needs to be mexed)

% Load image/label data
label = sign(double(imread('misc/X.png'))-1);
label = label(:,:,1);
noisy = label+randn(32,32);
nRows = 32;
nCols = 32;

if 0
    % Replicate
    rowRep = 4;
    colRep = 4;
    label = repmat(label,[rowRep rowRep]);
    imshow(label,[]);
    noisy = label+randn(size(label));
    nRows = 32*rowRep;
    nCols = 32*rowRep;
end

figure(1);
subplot(1,2,1);
imshow(label,[]);
title('Labels');
subplot(1,2,2);
imshow(noisy,[]);
title('Observed');
nInstances = 1;
nStates = 2;
drawnow;

% Convert to UGM feature/label format
nNodes = nRows*nCols;
X = reshape(noisy,[nInstances 1 nNodes]);
y = reshape(label,[nInstances nNodes]);

% Standardize Features
X = UGM_standardizeCols(X,tied);

% Convert from {-1,1} to {1,2} label representation
y(y==1) = 2;
y(y==-1) = 1;

% Make adjacency matrix
adjMatrix = fixed_Lattice(nRows,nCols);

% Make edges from adjacency matrix
edgeStruct=UGM_makeEdgeStruct(adjMatrix);

% Make edge features
Xedge = UGM_makeEdgeFeatures(X,edgeStruct.edgeEnds);
nEdges = edgeStruct.nEdges;

% Add bias to each node and edge
X = [ones(nInstances,1,nNodes) X];
Xedge = [ones(nInstances,1,nEdges) Xedge];

% Make Infostruct
infoStruct = UGM_makeInfoStruct(X,Xedge,edgeStruct,nStates,ising,tied,useMex,maxIter);

% Initialize weights
[w,v] = UGM_initWeights(infoStruct,'random');


% ================= Decoding ================================

figure(2);

% Make Potentials for first example with initial weights
nodePot = UGM_makeNodePotentials(X(1,:,:),w,infoStruct);
edgePot = UGM_makeEdgePotentials(Xedge(1,:,:),v,edgeStruct,infoStruct);

% Do ICM Decoding
y_ICM = UGM_Decode_ICM(nodePot,edgePot,edgeStruct,infoStruct);

% Show Results
figure(2);
subplot(2,2,1);
imshow(reshape(y_ICM,[nRows nCols]),[]);
title('ICM Decoding (rand params)');

if doGibbs
    % Do Gibbs-Sample Decoding
    y_Gibbs = UGM_Decode_Sample_Gibbs(nodePot,edgePot,edgeStruct,infoStruct);
    subplot(2,2,2);
    imshow(reshape(y_Gibbs,[nRows nCols]),[]);
    title('Gibbs Decoding (rand params)');
end

if doLoopy
    % Do Loopy Decoding
    y_Loopy = UGM_Decode_LBP(nodePot,edgePot,edgeStruct,infoStruct);
    subplot(2,2,3);
    imshow(reshape(y_Loopy,[nRows nCols]),[]);
    title('Loopy Decoding (rand params)');
end

% ===========================================================

% ================= Learning ================================

if doPseudo
    % Learn with Pseudolikelihood
    tic
    wv = minFunc(@UGM_PseudoLoss,[w;v],[],X,Xedge,y,edgeStruct,infoStruct);
    toc
elseif doViterbi
    % Learn with Viterbi Approximation (ICM used for decoding)
    wv = minFunc(@UGM_Loss,[w;v],[],X,Xedge,y,edgeStruct,infoStruct,@UGM_Infer_ViterbiApx,@UGM_Decode_ICM);
elseif doLoopyLearn
    % Learn with Loopy Approximation
    wv = minFunc(@UGM_Loss,[w;v],[],X,Xedge,y,edgeStruct,infoStruct,@UGM_Infer_LBP)
end
[w,v] = UGM_splitWeights(wv,infoStruct);

% ================ Decoding w/ Learned Parameters ==========

figure(3);

% Do Median Filtering
subplot(2,2,4);
imshow(medfilt2(noisy)>0,[]);
title('Median Filtering');

% Do Logistic Regression
subplot(2,2,2);
nFeatures = size(X,2);
X_logreg = zeros(0,nFeatures);
y_logreg = zeros(0,1);
for i = 1:nInstances
    for n = 1:nNodes
        X_logreg(i + nInstances*(n-1),:) = X(i,:,n);
        y_logreg(i + nInstances*(n-1),1) = y(i,n);
    end
end
w_logreg = minFunc(@SoftmaxLoss2,zeros(nFeatures,1),[],X_logreg,y_logreg,2);
imshow(reshape(1./exp(X_logreg*w_logreg) > .5,[nRows nCols]),[]);
title('Logistic Regression');

% Make Potentials for first example with initial weights
nodePot = UGM_makeNodePotentials(X(1,:,:),w,infoStruct);
edgePot = UGM_makeEdgePotentials(Xedge(1,:,:),v,edgeStruct,infoStruct);

% Do ICM Decoding
tic
y_ICM = UGM_Decode_ICM(nodePot,edgePot,edgeStruct,infoStruct);
toc
subplot(2,2,1);
imshow(reshape(y_ICM,[nRows nCols]),[]);
title('ICM Decoding (learned params)');

% Do Gibbs-Sample Decoding
if doGibbs
    y_Gibbs = UGM_Decode_ICM(nodePot,edgePot,edgeStruct,infoStruct);
    subplot(2,2,3);
    imshow(reshape(y_Gibbs,[nRows nCols]),[]);
    title('Gibbs Decoding (learned params)');
end

% Do Loopy Decoding
if doLoopy
    y_Loopy = UGM_Decode_LBP(nodePot,edgePot,edgeStruct,infoStruct);
    subplot(2,2,3);
    imshow(reshape(y_Loopy,[nRows nCols]),[]);
    title('Loopy Decoding (learned params)');
end



