clear all
close all

doGibbs = 0; % Slower than ICM
doLoopy = 0; % Slower than Gibbs

% Choose 1 of the below for learning strategy
doPseudo = 0;
doViterbi = 0; % Slower than Pseudolikelihood (should use a stochastic optimizer)
doLoopyLearn = 1; % Much Slower than Pseudolikelihood (loopy needs to be mexed)

% Load image/label data
label = sign(double(imread('misc/X.png'))-1);
label = label(:,:,1);
noisy = label+randn(32,32)/2;
figure(1);
subplot(1,2,1);
imshow(label,[]);
title('Labels');
subplot(1,2,2);
imshow(noisy,[]);
title('Observed');
nInstances = 1;
nStates = 2;
drawnow;

% Convert to UGM feature/label format
nRows = 32;
nCols = 32;
nNodes = nRows*nCols;
X = reshape(noisy,[nInstances 1 nNodes]);
y = reshape(label,[nInstances 1 nNodes]);

% Standardize Features
X = UGM_standardizeCols(X,1);

% Convert from {-1,1} to {1,2} label representation
y(y==1) = 2;
y(y==-1) = 1;

% Make adjacency matrix
adjMatrix = fixed_Lattice(nRows,nCols);

% Make edges from adjacency matrix
edgeStruct=UGM_makeEdgeStruct(adjMatrix);

% Make edge features
Xedge = UGM_makeEdgeFeaturesInvAbsDif(X,edgeStruct.edgeEnds);
nEdges = edgeStruct.nEdges;

% Add bias to each node and edge
X = [ones(nInstances,1,nNodes) X];
Xedge = [ones(nInstances,1,nEdges) Xedge];

% Make Infostruct
ising = 1;
tied = 1;
useMex = 1;
maxIter = 50;
infoStruct = UGM_makeInfoStruct(X,Xedge,edgeStruct,nStates,ising,tied,useMex,maxIter);

% Initialize weights
[w,v] = UGM_initWeights(infoStruct,'random');


% ================= Decoding ================================

figure(2);

% Make Potentials for first example with initial weights
nodePot = UGM_makeNodePotentials(X(1,:,:),w,infoStruct);
edgePot = UGM_makeEdgePotentials(Xedge(1,:,:),v,edgeStruct,infoStruct);

% Do ICM Decoding
y_ICM = UGM_Decode_ICM(nodePot,edgePot,edgeStruct,infoStruct);
subplot(2,2,1);
imshow(reshape(y_ICM,[nRows nCols]),[]);
title('ICM Decoding (rand params)');

if doGibbs
    % Do Gibbs-Sample Decoding
    y_Gibbs = UGM_Decode_Sample_Gibbs(nodePot,edgePot,edgeStruct,infoStruct);
    subplot(2,2,2);
    imshow(reshape(y_Gibbs,[nRows nCols]),[]);
    title('Gibbs Decoding (rand params)');
end

if doLoopy
    % Do Loopy Decoding
    y_Loopy = UGM_Decode_LBP(nodePot,edgePot,edgeStruct,infoStruct);
    subplot(2,2,3);
    imshow(reshape(y_Loopy,[nRows nCols]),[]);
    title('Loopy Decoding (rand params)');
end

% ===========================================================

% ================= Learning ================================

% Learn with Pseudolikelihood (note: gradient not implemented, so need to
% use finite differences)

UB = inf(size([w;v]));
LB = [-inf(size(w));zeros(size(v))];
    options.DerivativeCheck = 'off';
    options.numDiff = 0;
if doPseudo
    wv = minFuncBC(@UGM_PseudoLoss,[w;v],LB,UB,[],X,Xedge,y,edgeStruct,infoStruct);
    wv = minFuncBC(@UGM_PseudoLossL2,[w;v],LB,UB,[],X,Xedge,y,edgeStruct,infoStruct,1e-5,0,1e-5);
    pause;
elseif doLoopyLearn
    w = zeros(size(w));
    v = zeros(size(v));
    wv = minFunc(@UGM_Loss,[w;v],options,X,Xedge,y,edgeStruct,infoStruct,@UGM_Infer_LBP)
    wv = minFuncBC(@UGM_Loss,[w;v],LB,UB,options,X,Xedge,y,edgeStruct,infoStruct,@UGM_Infer_LBP)
    wv = minFuncBC(@UGM_LossL2,[w;v],LB,UB,options,X,Xedge,y,edgeStruct,infoStruct,@UGM_Infer_LBP,1e-5,0,1e-5)
    pause;
end
[w,v] = UGM_splitWeights(wv,infoStruct);

% ================ Decoding w/ Learned Parameters ==========

figure(3);

% Make Potentials for first example with initial weights
nodePot = UGM_makeNodePotentials(X(1,:,:),w,infoStruct);
edgePot = UGM_makeEdgePotentials(Xedge(1,:,:),v,edgeStruct,infoStruct);

% Do ICM Decoding
y_ICM = UGM_Decode_ICM(nodePot,edgePot,edgeStruct,infoStruct);
subplot(2,2,1);
imshow(reshape(y_ICM,[nRows nCols]),[]);
title('ICM Decoding (learned params)');

% Do Gibbs-Sample Decoding
if doGibbs
    y_Gibbs = UGM_Decode_ICM(nodePot,edgePot,edgeStruct,infoStruct);
    subplot(2,2,2);
    imshow(reshape(y_Gibbs,[nRows nCols]),[]);
    title('Gibbs Decoding (learned params)');
end

% Do Loopy Decoding
if doLoopy
    y_Loopy = UGM_Decode_LBP(nodePot,edgePot,edgeStruct,infoStruct);
    subplot(2,2,3);
    imshow(reshape(y_Loopy,[nRows nCols]),[]);
    title('Loopy Decoding (learned params)');
end

% Do Median Filtering
subplot(2,2,4);
imshow(medfilt2(noisy)>0,[]);
title('Median Filtering');
