function [p] = randomProject(c,weights,lambda)
% Finds solution p of:
%   min_p ||c - p||_2
%    s.t. |diag(weights)*p| <= lambda
%
% Assumes all elements of c are positive (this version handles ties)

nVars = length(c);

if sum(c.*weights) <= lambda || all(weights == 0 | c == 0)
    p = c;
    return;
end

mink = 1;
a = c(c>0 & weights>0);
w = weights(c>0 & weights>0);
q = a./w;
nVars = length(a);
offsetA = 0;
offsetB = 0;
while 1

    % Chose a random element of the partition
    ind = ceil(rand*length(a));
    a_k = a(ind);
    w_k = w(ind);
    q_k = q(ind);
    
    % Partition Elements
    lowerInd = find(q > q_k);
    middleInd = find(q == q_k);
    upperInd = find(q < q_k);

    % Find out what k value this element corresponds to
    k = length(lowerInd)+length(middleInd)+mink-1;
    
    % Compute running sum from 1 up to k-1
    s1A = offsetA + sum(a(lowerInd).*w(lowerInd)) + a_k*w_k*(length(middleInd)-1);
    s1B = offsetB + sum(w(lowerInd).^2) + w_k*w_k*(length(middleInd)-1);

    % Compute Soft-Threshold up to k
    LHS = s1A - q_k*s1B;

    if k < nVars
        % Find element k+1
        if isempty(upperInd)
            q_kP1 = q_maxkP1;
        else
            q_kP1 = max(q(upperInd));
        end
    else
        % We pad the end of the array with an extra '0' element
        q_kP1 = 0;
    end

    % Compute Soft-Threshold up to k+1
    s2A = s1A + a_k*w_k;
    s2B = s1B + w_k*w_k;
    RHS = s2A - q_kP1*s2B;

    % Check Termination Condition
    if lambda >= LHS && lambda < RHS
        break;
    end

    if lambda < LHS % Decrease maxk
        q_maxkP1 = q_kP1;
        a = a(lowerInd);
        w = w(lowerInd);
        q = q(lowerInd);
    else % lambda > RHS, Increase mink
        mink = k+1;
        offsetA = s2A;
        offsetB = s2B;
        a = a(upperInd);
        w = w(upperInd);
        q = q(upperInd);
    end
end

eta = (lambda - LHS)/s2B;
tau = q_k - eta;
p = max(c-tau*weights,0);